//   Copyright 2012 Georg-August-Universität Göttingen, Germany
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package de.ugoe.cs.autoquest.usability.rules.metrics;

import static de.ugoe.cs.autoquest.usability.tasktree.filters.EventTypeFilter.TEXT_INPUT;
import static de.ugoe.cs.autoquest.usability.util.TextInputUtil.aggregateEnteredTextFromTextInputs;
import static de.ugoe.cs.autoquest.usability.util.TextInputUtil.characterIsLetterOrDigitPredicate;

import java.util.List;

import com.google.common.base.CharMatcher;
import com.google.common.base.Optional;
import com.google.common.collect.Multiset;

import de.ugoe.cs.autoquest.tasktrees.treeifc.ITask;
import de.ugoe.cs.autoquest.tasktrees.treeifc.ITaskModel;
import de.ugoe.cs.autoquest.usability.EvaluationMethodCaller;
import de.ugoe.cs.autoquest.usability.result.DefectDescriptionResolver;
import de.ugoe.cs.autoquest.usability.result.UsabilityDefect;
import de.ugoe.cs.autoquest.usability.rules.UsabilityMetric;
import de.ugoe.cs.autoquest.usability.rules.UsabilityRule;
import de.ugoe.cs.autoquest.usability.tasktree.FilterResult;
import de.ugoe.cs.autoquest.usability.tasktree.IterativeDFSFilterStrategy;
import de.ugoe.cs.autoquest.usability.tasktree.filters.TaskModelFilter;

/**
 * <p>
 * TODO comment
 * </p>
 * 
 * @author Alexander Deicke
 */
public class NoLetterOrDigitRatioMetric extends UsabilityRule implements UsabilityMetric {

    /**
     * <p>
     * TODO: comment
     * </p>
     *
     * @param taskTree
     */
    public NoLetterOrDigitRatioMetric(ITaskModel taskModel) {
        super(taskModel);
        this.name = "NoLetterOrDigitRatio";
        this.defect = new DefectDescriptionResolver().descriptionFor(this.getClass().getSimpleName());
    }

    /* (non-Javadoc)
     * @see de.ugoe.cs.autoquest.usability.rules.UsabilityRule#check()
     */
    @Override
    public Optional<UsabilityDefect> calculate() {
        FilterResult textInputEvents = extractNodesFromTaskTree();
        float evaluationMetric = calculateEvaluationMetric(textInputEvents.tasksMatchedFilter());
        return this.defect.isPresent(evaluationMetric);
    }
    
    private FilterResult extractNodesFromTaskTree() {
        return new TaskModelFilter(new IterativeDFSFilterStrategy()).filterByEventType(TEXT_INPUT)
            .from(this.taskModel);
    }
    
    private float calculateEvaluationMetric(List<ITask> textInputEvents) {
        Multiset<String> enteredTextFragments =
            aggregateEnteredTextFromTextInputs(textInputEvents);
        int allCharactersCount = 0;
        int noLetterOrDigitCount = 0;
        for (String textFragment : enteredTextFragments.elementSet()) {
            int occurencesOfTextFragment = enteredTextFragments.count(textFragment);
            allCharactersCount += CharMatcher.ANY.countIn(textFragment) * occurencesOfTextFragment;
            noLetterOrDigitCount +=
                CharMatcher.forPredicate(characterIsLetterOrDigitPredicate()).countIn(textFragment) *
                    occurencesOfTextFragment;
        }
        return allCharactersCount != 0 ? (float) noLetterOrDigitCount / (float) allCharactersCount
            : 0;
    }

    /* (non-Javadoc)
     * @see de.ugoe.cs.autoquest.usability.rules.UsabilityRule#callEvaluationMetho(de.ugoe.cs.autoquest.usability.EvaluationMethodCaller)
     */
    @Override
    public Optional<UsabilityDefect> callEvaluationMethod(EvaluationMethodCaller evaluationMethodCaller)
    {
        return evaluationMethodCaller.check(this);
    }

}
